///////////////////////////////////////////////
//// XTR.AnsiString.cpp
//// (C) 2006 James Bernard Schumacher III
///////////////////////////////////////////////

#include <windows.h>
#include "XTR.String.hpp"

namespace XTR
{

AnsiString::AnsiString() throw() : m_pString(NULL),m_dwBufferSize(0)
{
    m_pString = new char[16];
    if (m_pString != NULL)
    {
        m_dwBufferSize = 16;
        m_pString[0] = 0;
    }
}

AnsiString::AnsiString(unsigned long dwBufferSize) throw() : m_pString(NULL),
     m_dwBufferSize(0)
{
    m_pString = new char[dwBufferSize];
    if (m_pString != NULL)
    {
        m_dwBufferSize = dwBufferSize;
        m_pString[0] = 0; 
    }
}

AnsiString::AnsiString(char ch) throw() : m_pString(NULL),m_dwBufferSize(0)
{
    m_pString = new char[16];
    if (m_pString != NULL)
    {
        m_dwBufferSize = 16;
        m_pString[0] = ch;
        m_pString[1] = 0;
    }
}

AnsiString::AnsiString(const char * pString) throw() : m_pString(NULL),m_dwBufferSize(0)
{
    const unsigned long dwLength = pString != NULL ? StringLength(pString) : 0;
    const unsigned long dwAllocate = DetermineBufferSize(dwLength);
    m_pString = new char[dwAllocate];
    if (m_pString != NULL)
    {
        m_dwBufferSize = dwAllocate;
        StringCopy(m_pString,pString,dwLength);
        m_pString[dwLength] = 0;
    }
}

AnsiString::AnsiString(const AnsiString & str) throw() : m_pString(NULL),m_dwBufferSize(0)
{
    const unsigned long dwLength = str.GetLength();
    const unsigned long dwAllocate = DetermineBufferSize(dwLength);
    m_pString = new char[dwAllocate];
    if (m_pString != NULL)
    {
        m_dwBufferSize = dwAllocate;
        StringCopy(m_pString,str.m_pString,dwLength);
        m_pString[dwLength] = 0;
    }
}

AnsiString::~AnsiString() throw()
{
    if (m_pString != NULL)
    {
        delete [] m_pString;
    }
}

bool AnsiString::IsEmpty() const throw()
{
    return m_pString == NULL || m_pString[0] == 0;
}

void AnsiString::Clear() throw()
{
    if (m_pString != NULL)
    {
        m_pString[0] = 0;
    }
}

unsigned long AnsiString::GetLength() const throw()
{
    return m_pString != NULL ? StringLength(m_pString) : 0;
}

unsigned long AnsiString::GetCapacity() const throw()
{
    return m_dwBufferSize;
}

bool AnsiString::SetCapacity(unsigned long dwSize,bool bKeepData) throw()
{
    bool bRet(false);
    if (dwSize > m_dwBufferSize)
    {
        char * pNewData = new char[dwSize];
        if (pNewData != NULL)
        {
            if (bKeepData == true)
            {
                const unsigned long dwLength = GetLength();
                StringCopy(pNewData,m_pString,dwLength);
                pNewData[dwLength] = 0;
            }
            else
            {
                pNewData[0] = 0;
            }
            if (m_pString != NULL)
            {
                delete [] m_pString;
            }
            m_pString = pNewData;
            m_dwBufferSize = dwSize;
            bRet = true;
        }
    }
    else
    {
        bRet = true;
    }
    return bRet;
}

const char * AnsiString::GetString() const throw()
{
    return m_pString;
}

char * AnsiString::GetBuffer() throw()
{
    return m_pString;
}

char * AnsiString::GetBuffer(unsigned long dwBufferSize) throw()
{
    if (SetCapacity(dwBufferSize,true) == false)
    {
        return NULL;
    }
    else
    {
        return m_pString;
    }
}

bool AnsiString::SetAt(unsigned long dwIndex,char ch) throw()
{
    const unsigned long dwLength = GetLength();
    if (dwIndex < dwLength)
    {
        m_pString[dwIndex] = ch;
        return true;
    }
    else
    {
        return false;
    }
}

bool AnsiString::GetAt(unsigned long dwIndex,char & chRef) const throw()
{
    const unsigned long dwLength = GetLength();
    if (dwIndex < dwLength)
    {
        chRef = m_pString[dwIndex];
        return true;
    }
    else
    {
        return false;
    }
}

bool AnsiString::Append(char ch) throw()
{
    if (ch != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + 1;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwBufferSize = DetermineBufferSize(dwNewLength);
            if (SetCapacity(dwBufferSize,true) == false)
            {
                return false;
            }
        }
        m_pString[dwLength] = ch;
        m_pString[dwNewLength] = 0;
        return true;
    }
    else
    {
        return false;
    }
}

bool AnsiString::Append(char ch,unsigned long dwCount) throw()
{
    bool bRet(false);
    if (ch != 0 && dwCount != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwCount;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAlloc = DetermineBufferSize(dwNewLength);
            if (SetCapacity(dwAlloc,true) == false)
            {
                return bRet;
            }
        }
        for (unsigned long i = 0; i < dwCount; ++i)
        {
            m_pString[dwLength + i] = ch;
        }
        bRet = true;
    }
    return bRet;
}

bool AnsiString::Append(const char * pString) throw()
{
     bool bRet(false);
     const unsigned long dwAddLength = pString != NULL ? StringLength(pString) : 0;
     if (dwAddLength != 0)
     {
         const unsigned long dwLength = GetLength();
         const unsigned long dwNewLength = dwLength + dwAddLength;
         if (dwNewLength >= m_dwBufferSize)
         {
             const unsigned long dwAlloc = DetermineBufferSize(dwNewLength);
             if (SetCapacity(dwAlloc,true) == false)
             {
                 return bRet;
             }
         }
         StringCopy(&m_pString[dwLength],pString,dwAddLength);
         m_pString[dwNewLength] = 0;
         bRet = true;
     }
     return bRet;
}

bool AnsiString::Append(const char * pString,unsigned long dwCharCount) throw()
{
     bool bRet(false);
     if (pString != NULL)
     {
         const unsigned long dwCopy = dwCharCount != 0xFFFFFFFF ? dwCharCount : StringLength(pString);
         const unsigned long dwLength = GetLength();
         const unsigned long dwNewLength = dwLength + dwCopy;
         if (dwNewLength >= m_dwBufferSize)
         {
             const unsigned long dwBufferSize = DetermineBufferSize(dwNewLength);
             if (SetCapacity(dwBufferSize,true) == false)
             {
                 return bRet;
             }
         }
         StringCopy(&m_pString[dwLength],pString,dwCopy);
         m_pString[dwNewLength] = 0;
         bRet = true;
     }
     return bRet;
}

bool AnsiString::Append(const AnsiString & str) throw()
{
    bool bRet(false);
    const unsigned long dwAddLength = str.GetLength();
    if (dwAddLength != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwAddLength;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAlloc = DetermineBufferSize(dwNewLength);
            if (SetCapacity(dwAlloc,true) == false)
            {
                return bRet;
            }
        }
        StringCopy(&m_pString[dwLength],str.m_pString,dwAddLength);
        m_pString[dwNewLength] = 0;
        bRet = true;
    }
    return bRet;
}

bool AnsiString::Append(const AnsiString & str,unsigned long dwCharCount) throw()
{
    bool bRet(false);
    const unsigned long dwTempAddLen = str.GetLength();
    unsigned long dwAddLength(0);
    if (dwCharCount == 0 || dwCharCount >= dwTempAddLen)
    {
        dwAddLength = dwTempAddLen;
    }
    else
    {
        dwAddLength = dwCharCount;
    }
    
    if (dwAddLength != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwAddLength;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAlloc = DetermineBufferSize(dwNewLength);
            if (SetCapacity(dwAlloc,true) == false)
            {
                return bRet;
            }
        }
        StringCopy(&m_pString[dwLength],str.m_pString,dwAddLength);
        m_pString[dwNewLength] = 0;
        bRet = true;
    }
    return bRet;
}

bool AnsiString::Append(const AnsiString * pStrings,unsigned long dwStringCount) throw()
{
    bool bRet(false);
    if (pStrings != NULL && dwStringCount != 0)
    {
        unsigned long * pdwLengths = new unsigned long[dwStringCount];
        if (pdwLengths != NULL)
        {
            unsigned long dwAddLength(0);
            for (unsigned long x = 0; x < dwStringCount; ++x)
            {
                pdwLengths[x] = pStrings[x].GetLength();
                dwAddLength += pdwLengths[x];
            }
            const unsigned long dwLength = GetLength();
            const unsigned long dwNewLength = dwAddLength + dwLength;
            if (dwNewLength >= m_dwBufferSize)
            {
                const unsigned long dwAllocate = DetermineBufferSize(dwNewLength);
                if (SetCapacity(dwAllocate,true) == false)
                {
                    delete [] pdwLengths;
                    return bRet;
                }
            }
            unsigned long dwTempPos(0);
            for (unsigned long v = 0; x < dwStringCount; ++v)
            {
                StringCopy(&m_pString[dwLength + dwTempPos],pStrings[v].m_pString,pdwLengths[v]);
                dwTempPos += pdwLengths[x];
            }
            m_pString[dwNewLength] = 0;
            delete [] pdwLengths;
            bRet = true;
        }
    }
    return bRet;
}

bool AnsiString::Prepend(char ch) throw()
{
    bool bRet(false);
    if (ch != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength == 0)
        {
            m_pString[0] = ch;
            m_pString[1] = 0;
            bRet = true;
        }
        else
        {
            const unsigned long dwNewLength = dwLength + 1;
            if (dwNewLength >= m_dwBufferSize)
            {
                const unsigned long dwAllocate = DetermineBufferSize(dwNewLength);
                char * pNewData = new char[dwAllocate];
                if (pNewData != NULL)
                {
                    pNewData[0] = ch;
                    StringCopy(&pNewData[1],m_pString,dwLength);
                    pNewData[dwNewLength] = 0;
                    delete [] m_pString;
                    m_pString = pNewData;
                    m_dwBufferSize = dwAllocate;
                    bRet = true;
                }
            }
            else
            {
                for (unsigned long x = dwLength; x > 0; --x)
                {
                    m_pString[x] = m_pString[x - 1];
                }
                m_pString[0] = ch;
                m_pString[dwNewLength] = 0;
                bRet = true;
            }
        }
    }
    return bRet;
}

bool AnsiString::Prepend(char ch,unsigned long dwCount) throw()
{
     bool bRet(false);
     if (ch != 0 && dwCount != 0)
     {
         const unsigned long dwLength = GetLength();
         const unsigned long dwNewLength = dwLength + dwCount;
         if (dwNewLength >= m_dwBufferSize)
         {
             const unsigned long dwAllocate = DetermineBufferSize(dwNewLength);
             char * pNewData = new char[dwAllocate];
             if (pNewData != NULL)
             {
                 unsigned long x(0);
                 for (; x < dwCount; ++x)
                 {
                     pNewData[x] = ch;
                 }
                 for (unsigned long y = 0; y < dwLength; ++y,++x)
                 {
                     pNewData[x] = m_pString[y];
                 }
                 pNewData[dwNewLength] = 0;
                 if (m_pString != NULL)
                 {
                     delete [] m_pString;
                 }
                 m_pString = pNewData;
                 m_dwBufferSize = dwAllocate;
                 bRet = true;
             }
         }
         else
         {
             for (unsigned long i = dwNewLength,j = dwLength; j > 0; --i,--j)
             {
                 m_pString[i - 1] = m_pString[j - 1];
             }
             for (unsigned long x = 0; x < dwCount; ++x)
             {
                 m_pString[x] = ch;
             }
             m_pString[dwNewLength] = 0;
             bRet = true;
         }
     }
     return bRet;
}

bool AnsiString::Prepend(const char * pString) throw()
{
    bool bRet(false);
    const unsigned long dwPrependLen = pString != NULL ? StringLength(pString) : 0;
    if (dwPrependLen != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwPrependLen;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAllocate = DetermineBufferSize(dwNewLength);
            char * pNewData = new char[dwAllocate];
            if (pNewData != NULL)
            {
                unsigned long x(0);
                for (; x < dwPrependLen; ++x)
                {
                    pNewData[x] = pString[x];
                }
                for (unsigned long i = 0; i < dwLength; ++i,++x)
                {
                    pNewData[x] = m_pString[i];
                }
                pNewData[dwNewLength] = 0;
                if (m_pString != NULL)
                {
                    delete [] m_pString;
                }
                m_pString = pNewData;
                m_dwBufferSize = dwAllocate;
                bRet = true;
            }
        }
        else
        {
            for (unsigned long v = dwNewLength,z = dwLength; z > 0; --z,--v)
            {
                m_pString[v - 1] = m_pString[z - 1];
            }
            for (unsigned long t = 0; t < dwPrependLen; ++t)
            {
                m_pString[t] = pString[t];
            }
            m_pString[dwNewLength] = 0;
            bRet = true;
        }
    }
    return bRet;
}

bool AnsiString::Prepend(const char * pString,unsigned long dwCharCount) throw()
{
    bool bRet(false);
    if (pString != NULL)
    {
        unsigned long dwPrependLen(0);
        if (dwCharCount == 0 || dwCharCount == 0xFFFFFFFF)
        {
            dwPrependLen = StringLength(pString);
        }
        else
        {
            dwPrependLen = dwCharCount;
        }
        if (dwPrependLen != 0)
        {
            const unsigned long dwLength = GetLength();
            const unsigned long dwNewLength = dwLength + dwPrependLen;
            if (dwNewLength >= m_dwBufferSize)
            {
                const unsigned long dwAllocate = DetermineBufferSize(dwNewLength);
                char * pNewData = new char[dwAllocate];
                if (pNewData != NULL)
                {
                    unsigned long x(0);
                    for (; x < dwPrependLen; ++x)
                    {
                        pNewData[x] = pString[x];
                    }
                    for (unsigned long y = 0; y < dwLength; ++y,++x)
                    {
                        pNewData[x] = pString[y];
                    }
                    pNewData[dwNewLength] = 0;
                    if (m_pString != NULL)
                    {
                        delete [] m_pString;
                    }
                    m_pString = pNewData;
                    m_dwBufferSize = dwAllocate;
                    bRet = true;
                }
            }
            else
            {
                for (unsigned long z = dwNewLength,v = dwLength; v > 0; --z,--v)
                {
                    m_pString[z - 1] = m_pString[v - 1];
                }
                for (unsigned long t = 0; t < dwNewLength; ++t)
                {
                    m_pString[t] = pString[t];
                }
                m_pString[dwNewLength] = 0;
                bRet = true;
            }
        }
    }
    return bRet;
}

bool AnsiString::Prepend(const AnsiString & str) throw()
{
    bool bRet(false);
    const unsigned long dwPrependLen = str.GetLength();
    if (dwPrependLen != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwPrependLen;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAllocate = DetermineBufferSize(dwNewLength);
            char * pNewData = new char[dwAllocate];
            if (pNewData != NULL)
            {
                unsigned long x(0);
                for (; x < dwPrependLen; ++x)
                {
                    pNewData[x] = str.m_pString[x];
                }
                for (unsigned long i = 0; i < dwLength; ++i,++x)
                {
                    pNewData[x] = m_pString[i];
                }
                pNewData[dwNewLength] = 0;
                if (m_pString != NULL)
                {
                    delete [] m_pString;
                }
                m_pString = pNewData;
                m_dwBufferSize = dwAllocate;
                bRet = true;
            }
        }
        else
        {
            for (unsigned long z = dwNewLength,v = dwLength; v > 0; --v,--z)
            {
                m_pString[z - 1] = m_pString[v - 1];
            }
            for (unsigned long t = 0; t < dwPrependLen; ++t)
            {
                m_pString[t] = str.m_pString[t];
            }
            m_pString[dwNewLength] = 0;
            bRet = true;
        }
    }
    return bRet;
}

bool AnsiString::Prepend(const AnsiString & str,unsigned long dwCharCount) throw()
{
    bool bRet(false);
    const unsigned long dwTempLen = str.GetLength();
    unsigned long dwPrependLen(0);
    if (dwCharCount == 0 || dwCharCount > dwTempLen)
    {
        dwPrependLen = dwTempLen;
    }
    else
    {
        dwPrependLen = dwCharCount;
    }
    
    if (dwPrependLen != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwPrependLen;
        const unsigned long dwTempAlloc = DetermineBufferSize(dwNewLength);
        const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
        char * pNewData = new char[dwAlloc];
        if (pNewData != NULL)
        {
            StringCopy(pNewData,str.m_pString,dwPrependLen);
            StringCopy(&pNewData[dwPrependLen],m_pString,dwLength);
            pNewData[dwNewLength] = 0;
            if (m_pString != NULL)
            {
                delete [] m_pString;
            }
            m_pString = pNewData;
            m_dwBufferSize = dwAlloc;
            bRet = true;
        }
    }
    return bRet;
}

bool AnsiString::Prepend(const AnsiString * pStrings,unsigned long dwStringCount) throw()
{
    bool bRet(false);
    if (pStrings != NULL && dwStringCount != 0)
    {
        unsigned long * pdwLengths = new unsigned long[dwStringCount];
        if (pdwLengths != NULL)
        {
            unsigned long dwPrependLen(0);
            for (unsigned long x = 0; x < dwStringCount; ++x)
            {
                pdwLengths[x] = pStrings[x].GetLength();
                dwPrependLen += pdwLengths[x];
            }
            const unsigned long dwLength = GetLength();
            const unsigned long dwNewLength = dwLength + dwPrependLen;
            if (dwNewLength >= m_dwBufferSize)
            {
                const unsigned long dwAllocate = DetermineBufferSize(dwNewLength);
                char * pNewData = new char[dwAllocate];
                if (pNewData != NULL)
                {
                    unsigned long xt(0);
                    for (unsigned long v = 0; v < dwStringCount; ++v)
                    {
                        StringCopy(&pNewData[xt],pStrings[v].m_pString,pdwLengths[v]);
                        xt += pdwLengths[v];
                    }
                    StringCopy(&pNewData[xt],m_pString,dwLength);
                    pNewData[dwNewLength] = 0;
                    if (m_pString != NULL)
                    {
                        delete [] m_pString;
                    }
                    m_pString = pNewData;
                    m_dwBufferSize = dwAllocate;
                    bRet = true;
                }
                else
                {
                    delete [] pdwLengths;
                    return bRet;
                }
            }
            else
            {
                for (unsigned long z = dwNewLength,v = dwLength; v > 0; --v,--z)
                {
                    m_pString[z - 1] = m_pString[v - 1];
                }
                unsigned long vx(0);
                for (unsigned long it = 0; it < dwStringCount; ++it)
                {
                    StringCopy(&m_pString[vx],pStrings[it].m_pString,pdwLengths[it]);
                    vx += pdwLengths[it];
                }
                m_pString[dwNewLength] = 0;
                bRet = true;
            }
            delete [] pdwLengths;
        }
    }
    return bRet;
}

bool AnsiString::Insert(unsigned long dwIndex,char ch) throw()
{
    bool bRet(false);
    if (ch != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + 1;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAlloc = DetermineBufferSize(dwNewLength);
            char * pNewData = new char[dwAlloc];
            if (pNewData != NULL)
            {
                unsigned long x(0);
                for (; x < dwIndex; ++x)
                {
                    pNewData[x] = m_pString[x];
                }
                unsigned long y = x++;
                pNewData[y] = ch;
                for (; y < dwLength; ++y,++x)
                {
                    pNewData[x] = m_pString[y];
                }
                pNewData[dwNewLength] = 0;
                if (m_pString != NULL)
                {
                    delete [] m_pString;
                }
                m_pString = pNewData;
                m_dwBufferSize = dwAlloc;
                bRet = true;
            }
        }
        else
        {
            for (unsigned long v = dwLength; v > dwIndex; --v)
            {
                m_pString[v] = m_pString[v - 1];
            }
            m_pString[dwIndex] = ch;
            m_pString[dwNewLength] = 0;
            bRet = true;
        }
    }
    return bRet;
}

bool AnsiString::Insert(unsigned long dwIndex,char ch,unsigned long dwCount) throw()
{
    bool bRet(false);
    if (ch != 0 && dwCount != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwCount;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAllocate = DetermineBufferSize(dwNewLength);
            char * pNewData = new char[dwAllocate];
            if (pNewData != NULL)
            {
                unsigned long x(0);
                for (; x < dwIndex; ++x)
                {
                    pNewData[x] = m_pString[x];
                }
                unsigned long y = x++;
                pNewData[y] = ch;
                for (; y < dwLength; ++y,++x)
                {
                    pNewData[x] = m_pString[y];
                }
                pNewData[dwNewLength] = 0;
                if (m_pString != NULL)
                {
                    delete [] m_pString;
                }
                m_pString = pNewData;
                m_dwBufferSize = dwAllocate;
                bRet = true;
            }
        }
        else
        {
            if (dwIndex >= dwLength)
            {
                for (unsigned long vi = 0; vi < dwCount; ++vi)
                {
                    m_pString[dwLength + vi] = ch;
                }
                m_pString[dwNewLength] = 0;
            }
            else
            {
                for (unsigned long zt = dwNewLength,zu = dwLength; zu > dwIndex; --zu,--zt)
                {
                    m_pString[zt - 1] = m_pString[zu - 1];
                }
                for (unsigned long zx = 0; zx < dwCount; ++zx)
                {
                    m_pString[dwIndex + zx] = ch;
                }
                m_pString[dwNewLength] = 0;
            }
            bRet = true;
        }
    }
    return bRet;
}

bool AnsiString::Insert(unsigned long dwIndex,const char * pString) throw()
{
     bool bRet(false);
     const unsigned long dwInsertLen = pString != NULL ? StringLength(pString) : 0;
     if (dwInsertLen != 0)
     {
         const unsigned long dwLength = GetLength();
         const unsigned long dwNewLength = dwLength + dwInsertLen;
         if (dwNewLength >= m_dwBufferSize)
         {
             const unsigned long dwAllocate = DetermineBufferSize(dwNewLength);
             char * pNewData = new char[dwAllocate];
             if (pNewData != NULL)
             {
                 unsigned long x = 0;
                 for (; x < dwIndex; ++x)
                 {
                     pNewData[x] = m_pString[x];
                 }
                 unsigned long y = x;
                 for (unsigned long i = 0; i < dwInsertLen; ++i,++x)
                 {
                     pNewData[x] = pString[i];
                 }
                 for (; y < dwLength; ++y,++x)
                 {
                     pNewData[x] = m_pString[y];
                 }
                 pNewData[dwNewLength] = 0;
                 if (m_pString != NULL)
                 {
                     delete [] m_pString;
                 }
                 m_pString = pNewData;
                 m_dwBufferSize = dwAllocate;
                 bRet = true;
             }
         }
         else
         {
             if (dwIndex >= dwLength)
             {
                 for (unsigned long xt = 0; xt < dwInsertLen; ++xt)
                 {
                     m_pString[dwLength + xt] = pString[xt];
                 }
                 m_pString[dwNewLength] = 0;
             }
             else
             {
                 for (unsigned long zt = dwNewLength,zu = dwLength; zu > dwIndex; --zt,--zu)
                 {
                     m_pString[zt - 1] = m_pString[zu - 1];
                 }
                 for (unsigned long zd = 0; zd < dwInsertLen; ++zd)
                 {
                     m_pString[dwIndex + zd] = pString[zd];
                 }
                 m_pString[dwNewLength] = 0;
             }
             bRet = true;
         }
     }
     return bRet;
}

bool AnsiString::Insert(unsigned long dwIndex,const char * pString,unsigned long dwCharCount) throw()
{
     bool bRet(false);
     if (pString != NULL)
     {
         unsigned long dwInsertLen(0);
         if (dwCharCount == 0 || dwCharCount == 0xFFFFFFFF)
         {
             dwInsertLen = StringLength(pString);
         }
         else
         {
             dwInsertLen = dwCharCount;
         }
         
         if (dwInsertLen != 0)
         {
             const unsigned long dwLength = GetLength();
             const unsigned long dwNewLength = dwLength + dwInsertLen;
             if (dwNewLength >= m_dwBufferSize)
             {
                 const unsigned long dwBufferSize = DetermineBufferSize(dwNewLength);
                 char * pNewData = new char[dwBufferSize];
                 if (pNewData != NULL)
                 {
                     unsigned long x = 0;
                     for (; x < dwIndex; ++x)
                     {
                         pNewData[x] = m_pString[x];
                     }
                     unsigned long y = x;
                     for (unsigned long i = 0; i < dwInsertLen; ++i,++x)
                     {
                         pNewData[x] = pString[i];
                     }
                     for (; y < dwLength; ++y,++x)
                     {
                         pNewData[x] = m_pString[y];
                     }
                     pNewData[dwNewLength] = 0;
                     if (m_pString != NULL)
                     {
                         delete [] m_pString;
                     }
                     m_pString = pNewData;
                     m_dwBufferSize = dwBufferSize;
                     bRet = true;
                 }
             }
             else
             {
                 if (dwIndex >= dwLength)
                 {
                     for (unsigned long x = 0; x < dwInsertLen; ++x)
                     {
                         m_pString[dwLength + x] = pString[x];
                     }
                     m_pString[dwNewLength] = 0;
                 }
                 else
                 {
                     for (unsigned long zt = dwNewLength, zu = dwLength; zu > dwIndex; --zu,--zt)
                     {
                         m_pString[zt - 1] = m_pString[zu - 1];
                     }
                     for (unsigned long xt = 0; xt < dwInsertLen; ++xt)
                     {
                         m_pString[dwIndex + xt] = pString[xt];
                     }
                     m_pString[dwNewLength] = 0;
                 }
                 bRet = true;
             }
         }
     }
     return bRet;
}

bool AnsiString::Insert(unsigned long dwIndex,const AnsiString & str) throw()
{
     bool bRet(false);
     const unsigned long dwInsertLen = str.GetLength();
     if (dwInsertLen != 0)
     {
         const unsigned long dwLength = GetLength();
         const unsigned long dwNewLength = dwLength + dwInsertLen;
         const unsigned long dwTempAlloc = DetermineBufferSize(dwNewLength);
         const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
         char * pNewData = new char[dwAlloc];
         if (pNewData != NULL)
         {
             unsigned long x = 0;
             for (; x < dwIndex; ++x)
             {
                 pNewData[x] = m_pString[x];
             }
             unsigned long y = x;
             for (unsigned long i = 0; i < dwInsertLen; ++i,++x)
             {
                 pNewData[x] = str.m_pString[i];
             }
             for (; y < dwLength; ++y,++x)
             {
                 pNewData[x] = m_pString[y];
             }
             pNewData[dwNewLength] = 0;
             if (m_pString != NULL)
             {
                 delete [] m_pString;
             }
             m_pString = pNewData;
             m_dwBufferSize = dwAlloc;
             bRet = true;
         }
     }
     return bRet;
}

bool AnsiString::Insert(unsigned long dwIndex,const AnsiString & str,unsigned long dwCharCount) throw()
{
     bool bRet(false);
     const unsigned long dwTempLen = str.GetLength();
     unsigned long dwInsertLen(0);
     if (dwCharCount == 0 || dwCharCount > dwTempLen)
     {
         dwInsertLen = dwTempLen;
     }
     else
     {
         dwInsertLen = dwCharCount;
     }
     
     if (dwInsertLen != 0)
     {
         const unsigned long dwLength = GetLength();
         const unsigned long dwNewLength = dwLength + dwInsertLen;
         const unsigned long dwTempAlloc = DetermineBufferSize(dwNewLength);
         const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
         char * pNewData = new char[dwAlloc];
         if (pNewData != NULL)
         {
             unsigned long x = 0;
             for (; x < dwIndex; ++x)
             {
                 pNewData[x] = m_pString[x];
             }
             unsigned long y = x;
             for (unsigned long i = 0; i < dwInsertLen; ++i,++x)
             {
                 pNewData[x] = str.m_pString[i];
             }
             for (; y < dwLength; ++y,++x)
             {
                 pNewData[x] = m_pString[y];
             }
             pNewData[dwNewLength] = 0;
             if (m_pString != NULL)
             {
                 delete [] m_pString;
             }
             m_pString = pNewData;
             m_dwBufferSize = dwAlloc;
             bRet = true;
         }
     }
     return bRet;
}

bool AnsiString::Find(char chFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    if (chFind != 0)
    {
        const unsigned long dwLength = GetLength();
        for (unsigned long x = 0; x < dwLength; ++x)
        {
            if (m_pString[x] == chFind)
            {
                dwFoundIndex = x;
                bFound = true;
                break;
            }
        }
    }
    return bFound;
}

bool AnsiString::Find(const char * pstrFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const unsigned long dwFindLen = pstrFind != NULL ? StringLength(pstrFind) : 0;
    if (dwFindLen != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwFindLen <= dwLength)
        {
            const unsigned long dwLastPossibleIndex = dwLength - dwFindLen;
            
            for (unsigned long x = 0; x <= dwLastPossibleIndex; ++x)
            {
                bFound = true;
                for (unsigned long i = 0; i < dwFindLen; ++i)
                {
                    if (m_pString[x + i] != pstrFind[i])
                    {
                        bFound = false;
                        break;
                    }
                }
                if (bFound == true)
                {
                    dwFoundIndex = x;
                    break;
                }
            }
        }
    }
    return bFound;
}

bool AnsiString::Find(const AnsiString & strFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const unsigned dwFindLength = strFind.GetLength();
    if (dwFindLength != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwFindLength <= dwLength)
        {
            const unsigned long dwLastPossibleIndex = dwLength - dwFindLength;
            for (unsigned long x = 0; x <= dwLastPossibleIndex; ++x)
            {
                bFound = true;
                for (unsigned long i = 0; i < dwFindLength; ++i)
                {
                   if (m_pString[x + i] != strFind.m_pString[i])
                   {
                       bFound = false;
                       break;
                   }
                }
                
                if (bFound == true)
                {
                    dwFoundIndex = x;
                    break;
                }
            }
        }
    }
    return bFound;
}

bool AnsiString::FindFrom(unsigned long dwStart,char chFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    if (chFind != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwStart < dwLength)
        {
            for (unsigned long x = dwStart; x < dwLength; ++x)
            {
                if (m_pString[x] == chFind)
                {
                    dwFoundIndex = x;
                    bFound = true;
                    break;
                }
            }
        }
    }
    return bFound;
}

bool AnsiString::FindFrom(unsigned long dwStart,const char * pstrFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const unsigned long dwFindLen = pstrFind != NULL ? StringLength(pstrFind) : 0;
    if (dwFindLen != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwStart < dwLength)
        {
            const unsigned long dwCheckIndex = dwStart + dwFindLen;
            if (dwCheckIndex <= dwLength)
            {
                const unsigned long dwLastPossibleIndex = dwLength - dwFindLen;
                for (unsigned long x = dwStart; x <= dwLastPossibleIndex; ++x)
                {
                    bFound = true;
                    for (unsigned long i = 0; i < dwFindLen; ++i)
                    {
                        if (m_pString[x + i] != pstrFind[i])
                        {
                            bFound = false;
                            break;
                        }
                    }
                    
                    if (bFound == true)
                    {
                        dwFoundIndex = x;
                        break;
                    }
                }
            }
        }
    }
    return bFound;
}

bool AnsiString::FindFrom(unsigned long dwStart,const AnsiString & strFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const unsigned long dwFindLength = strFind.GetLength();
    if (dwFindLength != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwStart < dwLength)
        {
            const unsigned long dwCheckIndex = dwStart + dwFindLength;
            if (dwCheckIndex <= dwLength)
            {
                const unsigned long dwLastPossibleIndex = dwLength - dwFindLength;
                for (unsigned long x = dwStart; x <= dwLastPossibleIndex; ++x)
                {
                    bFound = true;
                    for (unsigned long i = 0; i < dwFindLength; ++i)
                    {
                        if (m_pString[x + i] != strFind.m_pString[i])
                        {
                            bFound = false;
                            break;
                        }
                    }
                    
                    if (bFound == true)
                    {
                        dwFoundIndex = x;
                        break;
                    }
                }
            }
        }
    }
    return bFound;
}

bool AnsiString::FindLastInstanceOf(char chFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    if (chFind != 0)
    {
        const unsigned long dwLength = GetLength();
        for (unsigned long x = dwLength; x > 0; --x)
        {
            if (m_pString[x - 1] == chFind)
            {
                dwFoundIndex = x - 1;
                bFound = true;
                break;
            }
        }
    }
    return bFound;
}

bool AnsiString::FindLastInstanceOf(const char * pstrFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const unsigned long dwFindLen = pstrFind != NULL ? StringLength(pstrFind) : 0;
    if (dwFindLen != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwFindLen <= dwLength)
        {
            const unsigned long dwFirstPossibleIndex = dwLength - dwFindLen;
            for (unsigned long x = dwFirstPossibleIndex; x < dwLength; --x)
            {
                bFound = true;
                for (unsigned long i = 0; i < dwFindLen; ++i)
                {
                    if (m_pString[x + i] != pstrFind[i])
                    {
                        bFound = false;
                        break;
                    }
                }
                
                if (bFound == true)
                {
                    dwFoundIndex = x;
                    break;
                }
            }
        }
    }
    return bFound;
}

bool AnsiString::FindLastInstanceOf(const AnsiString & strFind,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const unsigned long dwFindLength = strFind.GetLength();
    if (dwFindLength != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwFindLength <= dwLength)
        {
            const unsigned long dwFirstPossibleIndex = dwLength - dwFindLength;
            for (unsigned long x = dwFirstPossibleIndex; x < dwLength; --x)
            {
                bFound = true;
                for (unsigned long i = 0; i < dwFindLength; ++i)
                {
                    if (m_pString[x + i] != strFind.m_pString[i])
                    {
                        bFound = false;
                        break;
                    }
                }
                
                if (bFound == true)
                {
                    dwFoundIndex = x;
                    break;
                }
            }
        }
    }
    return bFound;
}

void AnsiString::FindAllInstancesOf(char chFind,std::vector<unsigned long> & arFoundIndices) const throw()
{
    arFoundIndices.clear();
    if (chFind != 0)
    {
        const unsigned long dwLength = GetLength();
        for (unsigned long i = 0; i < dwLength; ++i)
        {
            if (m_pString[i] == chFind)
            {
                arFoundIndices.push_back(i);
            }
        }
    }
}

void AnsiString::FindAllInstancesOf(const char * pstrFind,std::vector<unsigned long> & arFoundIndices) const throw()
{
    arFoundIndices.clear();
    const unsigned long dwFindLength = pstrFind != NULL ? StringLength(pstrFind) : 0;
    if (dwFindLength != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwFindLength <= dwLength)
        {
            bool bFound = false;
            const unsigned long dwLastPossibleIndex = dwLength - dwFindLength;
            unsigned long dwCurrentIndex(0);
            do
            {
                bFound = true;
                for (unsigned long i = 0; i < dwFindLength; ++i)
                {
                    if (m_pString[dwCurrentIndex + i] != pstrFind[i])
                    {
                        bFound = false;
                        break;
                    }
                }
                
                if (bFound == true)
                {
                    arFoundIndices.push_back(dwCurrentIndex);
                    dwCurrentIndex += dwFindLength;
                }
                else
                {
                    ++dwCurrentIndex;
                }
            } while (dwCurrentIndex <= dwLastPossibleIndex);
        }
    }
}

void AnsiString::FindAllInstancesOf(const AnsiString & strFind,std::vector<unsigned long> & arFoundIndices) const throw()
{
    arFoundIndices.clear();
    const unsigned long dwFindLength = strFind.GetLength();
    if (dwFindLength != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwFindLength <= dwLength)
        {
            bool bFound = false;
            const unsigned long dwLastPossibleIndex = dwLength - dwFindLength;
            unsigned long dwCurrentIndex(0);
            do
            {
                bFound = true;
                for (unsigned long i = 0; i < dwFindLength; ++i)
                {
                    if (m_pString[dwCurrentIndex + i] != strFind.m_pString[i])
                    {
                        bFound = false;
                        break;
                    }
                }
                
                if (bFound == true)
                {
                    arFoundIndices.push_back(dwCurrentIndex);
                    dwCurrentIndex += dwFindLength;
                }
                else
                {
                    ++dwCurrentIndex;
                }
            } while (dwCurrentIndex <= dwLastPossibleIndex);
        }
    }
}

bool AnsiString::FindOneOf(const char * pstrCharSet,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const AnsiString strTemp(pstrCharSet);
    const unsigned long dwCharSetLen = strTemp.GetLength();
    if (dwCharSetLen != 0)
    {
        const unsigned long dwLength = GetLength();
        unsigned long dwTempIndex(0);
        for (unsigned long i = 0; i < dwLength; ++i)
        {
            bFound = strTemp.Find(m_pString[i],dwTempIndex);
            if (bFound == true)
            {
                dwFoundIndex = i;
                break;
            }
        }
    }
    return bFound;
}

bool AnsiString::FindOneOf(const AnsiString & strCharSet,unsigned long & dwFoundIndex) const throw()
{
    bool bFound(false);
    const unsigned long dwCharSetLen = strCharSet.GetLength();
    if (dwCharSetLen != 0)
    {
        const unsigned long dwLength = GetLength();
        unsigned long dwTempIndex(0);
        for (unsigned long i = 0; i < dwLength; ++i)
        {
            bFound = strCharSet.Find(m_pString[i],dwTempIndex);
            if (bFound == true)
            {
                dwFoundIndex = i;
                break;
            }
        }
    }
    return bFound;
}

unsigned long AnsiString::Replace(char chOld,char chNew) throw()
{
    unsigned long dwReplacedCount(0);
    if (chOld != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength != 0)
        {
            if (chNew != 0)
            {
                for (unsigned long i = 0; i < dwLength; ++i)
                {
                    if (m_pString[i] == chOld)
                    {
                        m_pString[i] = chNew;
                        ++dwReplacedCount;
                    }
                }
            }
            else
            {
                char * pNewData = new char[m_dwBufferSize];
                if (pNewData != NULL)
                {
                    unsigned long dwBufferPos(0);
                    for (unsigned long x = 0; x < dwLength; ++x)
                    {
                        if (m_pString[x] != chOld)
                        {
                            pNewData[dwBufferPos] = m_pString[x];
                            ++dwBufferPos;
                        }
                        else
                        {
                            ++dwReplacedCount;
                        }
                    }
                    pNewData[dwBufferPos] = 0;
                    delete [] m_pString;
                    m_pString = pNewData;
                }
            }
        }
    }
    return dwReplacedCount;
}

unsigned long AnsiString::Replace(const char * pstrOld,const char * pstrNew) throw()
{
    unsigned long dwReplacedCount(0);
    const unsigned long dwOldLength = pstrOld != NULL ? StringLength(pstrOld) : 0;
    if (dwOldLength != 0)
    {
        const unsigned long dwLength = GetLength();
        std::vector<unsigned long> arFoundIndices;
        FindAllInstancesOf(pstrOld,arFoundIndices);
        const unsigned long dwFoundCount = arFoundIndices.size();
        if (dwFoundCount != 0)
        {
            const unsigned long dwNewLength = pstrNew != NULL ? StringLength(pstrNew) : 0;
            unsigned long dwAdjustment(0);
            bool bSubtract = false;
            if (dwNewLength > dwOldLength)
            {
                dwAdjustment = (dwNewLength - dwOldLength) * dwFoundCount;
            }
            else if (dwOldLength > dwNewLength)
            {
                dwAdjustment = (dwOldLength - dwNewLength) * dwFoundCount;
                bSubtract = true;
            }

            unsigned long dwNewTotalLength = dwLength;
            if (bSubtract == true)
            {
                dwNewTotalLength -= dwAdjustment;
            }
            else
            {
                dwNewTotalLength += dwAdjustment;
            }

            if (dwNewTotalLength != dwLength)
            {
                const unsigned long dwTempAlloc = DetermineBufferSize(dwNewTotalLength);
                const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
                char * pNewData = new char[dwAlloc];
                if (pNewData != NULL)
                {
                    unsigned long dwBufferPos(0),dwCurrentPos(0);
                    for (unsigned long i = 0; i < dwFoundCount; ++i)
                    {
                        for (; dwCurrentPos < arFoundIndices[i]; ++dwCurrentPos,++dwBufferPos)
                        {
                            pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                        }
                        for (unsigned long v = 0; v < dwNewLength; ++v,++dwBufferPos)
                        {
                            pNewData[dwBufferPos] = pstrNew[v];
                        }
                        dwCurrentPos += dwOldLength;
                    }
                    for (; dwCurrentPos < dwLength; ++dwCurrentPos,++dwBufferPos)
                    {
                        pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                    }
                    pNewData[dwNewTotalLength] = 0;
                    delete [] m_pString;
                    m_pString = pNewData;
                    m_dwBufferSize = dwAlloc;
                    dwReplacedCount = dwFoundCount;
                }
            }
            else
            {
                for (unsigned long i = 0; i < dwFoundCount; ++i)
                {
                    for (unsigned long x = 0; x < dwNewLength; ++x)
                    {
                        m_pString[arFoundIndices[i] + x] = pstrNew[x];
                    }
                }
                dwReplacedCount = dwFoundCount;
            }
        }
    }
    return dwReplacedCount;
}

unsigned long AnsiString::Replace(const AnsiString & strOld,const AnsiString & strNew) throw()
{
    unsigned long dwReplacedCount(0);
    const unsigned long dwOldLength = strOld.GetLength();
    if (dwOldLength != 0)
    {
        const unsigned long dwLength = GetLength();
        std::vector<unsigned long> arFoundIndices;
        FindAllInstancesOf(strOld,arFoundIndices);
        const unsigned long dwFoundCount = arFoundIndices.size();
        if (dwFoundCount != 0)
        {
            const unsigned long dwNewLength = strNew.GetLength();
            unsigned long dwAdjustment(0);
            bool bSubtract = false;
            if (dwNewLength > dwOldLength)
            {
                dwAdjustment = (dwNewLength - dwOldLength) * dwFoundCount;
            }
            else if (dwOldLength > dwNewLength)
            {
                dwAdjustment = (dwOldLength - dwNewLength) * dwAdjustment;
                bSubtract = true;
            }
            unsigned long dwNewTotalLength = dwLength;
            if (bSubtract == true)
            {
                dwNewTotalLength -= dwAdjustment;
            }
            else
            {
                dwNewTotalLength += dwAdjustment;
            }
            if (dwNewTotalLength != dwLength)
            {
                const unsigned long dwTempAlloc = DetermineBufferSize(dwNewTotalLength);
                const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
                char * pNewData = new char[dwAlloc];
                if (pNewData != NULL)
                {
                    unsigned long dwBufferPos(0),dwCurrentPos(0);
                    for (unsigned long t = 0; t < dwFoundCount; ++t)
                    {
                        for (; dwCurrentPos < arFoundIndices[t]; ++dwCurrentPos,++dwBufferPos)
                        {
                            pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                        }
                        for (unsigned long v = 0; v < dwNewLength; ++v,++dwBufferPos)
                        {
                            pNewData[dwBufferPos] = strNew.m_pString[v];
                        }
                    }
                    for (; dwCurrentPos < dwLength; ++dwCurrentPos,++dwBufferPos)
                    {
                        pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                    }
                    pNewData[dwNewTotalLength] = 0;
                    delete [] m_pString;
                    m_pString = pNewData;
                    m_dwBufferSize = dwAlloc;
                    dwReplacedCount = dwFoundCount;
                }
            }
            else
            {
                for (unsigned long i = 0; i < dwFoundCount; ++i)
                {
                    for (unsigned long x = 0; x < dwNewLength; ++x)
                    {
                        m_pString[arFoundIndices[i] + x] = strNew.m_pString[x];
                    }
                }
                dwReplacedCount = dwFoundCount;
            }
        }
    }
    return dwReplacedCount;
}

unsigned long AnsiString::Replace(const char * pstrOld,const char * pstrNew,bool bWholeWordOnly) throw()
{
    unsigned long dwReplacedCount(0);
    const unsigned long dwOldLength = pstrOld != NULL ? StringLength(pstrOld) : 0;
    if (dwOldLength != 0)
    {
        const unsigned long dwLength = GetLength();
        std::vector<unsigned long> arFoundIndices;
        FindAllInstancesOf(pstrOld,arFoundIndices);
        const unsigned long dwFoundCount = arFoundIndices.size();
        if (dwFoundCount != 0)
        {
            const unsigned long dwNewLength = pstrNew != NULL ? StringLength(pstrNew) : 0;
            unsigned long dwAdjustment(0);
            bool bSubtract = false;
            if (dwNewLength > dwOldLength)
            {
                dwAdjustment = (dwNewLength - dwOldLength) * dwFoundCount;
            }
            else if (dwOldLength > dwNewLength)
            {
                dwAdjustment = (dwOldLength - dwNewLength) * dwFoundCount;
                bSubtract = true;
            }

            unsigned long dwNewTotalLength = dwLength;
            if (bSubtract == true)
            {
                dwNewTotalLength -= dwAdjustment;
            }
            else
            {
                dwNewTotalLength += dwAdjustment;
            }

            if (dwNewTotalLength != dwLength)
            {
                const unsigned long dwTempAlloc = DetermineBufferSize(dwNewTotalLength);
                const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
                char * pNewData = new char[dwAlloc];
                if (pNewData != NULL)
                {
                    unsigned long dwBufferPos(0),dwCurrentPos(0);
                    bool bReplace = false;
                    for (unsigned long i = 0; i < dwFoundCount; ++i)
                    {
                        bReplace = false;
                        for (; dwCurrentPos < arFoundIndices[i]; ++dwCurrentPos,++dwBufferPos)
                        {
                            pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                        }
                        if (bWholeWordOnly == true)
                        {
                            if (dwCurrentPos != 0)
                            {
                                if (IsCharAlphaDigit(m_pString[dwCurrentPos - 1]) == false && m_pString[dwCurrentPos - 1] != '_')
                                {
                                    bReplace = true;
                                }
                            }
                            else if (IsCharAlphaDigit(m_pString[dwCurrentPos]) == false && m_pString[dwCurrentPos] != '_')
                            {
                                bReplace = true;
                            }
                        }
                        if (bReplace == true)
                        {
                            if (IsCharAlphaDigit(m_pString[dwCurrentPos + dwOldLength]) == true && m_pString[dwCurrentPos + dwOldLength] == '_')
                            {
                                bReplace = false;
                            }
                        }

                        if (bReplace == true)
                        {
                            for (unsigned long v = 0; v < dwNewLength; ++v,++dwBufferPos)
                            {
                                pNewData[dwBufferPos] = pstrNew[v];
                            }
                            dwCurrentPos += dwOldLength;
                        }
                        else
                        {
                            for (unsigned long it = 0; it < dwOldLength; ++it,++dwCurrentPos,++dwBufferPos)
                            {
                                pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                            }
                        }
                    }
                    for (; dwCurrentPos < dwLength; ++dwCurrentPos,++dwBufferPos)
                    {
                        pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                    }
                    pNewData[dwNewTotalLength] = 0;
                    delete [] m_pString;
                    m_pString = pNewData;
                    m_dwBufferSize = dwAlloc;
                    dwReplacedCount = dwFoundCount;
                }
            }
            else
            {
                for (unsigned long i = 0; i < dwFoundCount; ++i)
                {
                    for (unsigned long x = 0; x < dwNewLength; ++x)
                    {
                        m_pString[arFoundIndices[i] + x] = pstrNew[x];
                    }
                }
                dwReplacedCount = dwFoundCount;
            }
        }
    }
    return dwReplacedCount;
}

unsigned long AnsiString::Replace(const AnsiString & strOld,const AnsiString & strNew,bool bWholeWordOnly) throw()
{
    unsigned long dwReplacedCount(0);
    const unsigned long dwOldLength = strOld.GetLength();
    if (dwOldLength != 0)
    {
        const unsigned long dwLength = GetLength();
        std::vector<unsigned long> arFoundIndices;
        FindAllInstancesOf(strOld,arFoundIndices);
        const unsigned long dwFoundCount = arFoundIndices.size();
        if (dwFoundCount != 0)
        {
            const unsigned long dwNewLength = strNew.GetLength();
            unsigned long dwAdjustment(0);
            bool bSubtract = false;
            if (dwNewLength > dwOldLength)
            {
                dwAdjustment = (dwNewLength - dwOldLength) * dwFoundCount;
            }
            else if (dwOldLength > dwNewLength)
            {
                dwAdjustment = (dwOldLength - dwNewLength) * dwAdjustment;
                bSubtract = true;
            }
            unsigned long dwNewTotalLength = dwLength;
            if (bSubtract == true)
            {
                dwNewTotalLength -= dwAdjustment;
            }
            else
            {
                dwNewTotalLength += dwAdjustment;
            }
            if (dwNewTotalLength != dwLength)
            {
                const unsigned long dwTempAlloc = DetermineBufferSize(dwNewTotalLength);
                const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
                char * pNewData = new char[dwAlloc];
                if (pNewData != NULL)
                {
                    unsigned long dwBufferPos(0),dwCurrentPos(0);
                    bool bReplace = false;
                    for (unsigned long t = 0; t < dwFoundCount; ++t)
                    {
                        bReplace = false;
                        for (; dwCurrentPos < arFoundIndices[t]; ++dwCurrentPos,++dwBufferPos)
                        {
                            pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                        }
                        if (bWholeWordOnly == true)
                        {
                            if (dwCurrentPos != 0)
                            {
                                if (IsCharAlphaDigit(m_pString[dwCurrentPos - 1]) == false && m_pString[dwCurrentPos - 1] != '_')
                                {
                                    bReplace = true;
                                }
                            }
                            else if (IsCharAlphaDigit(m_pString[dwCurrentPos]) == false && m_pString[dwCurrentPos] != '_')
                            {
                                bReplace = true;
                            }
                        }
                        if (bReplace == true)
                        {
                            if (IsCharAlphaDigit(m_pString[dwCurrentPos + dwOldLength]) == true && m_pString[dwCurrentPos + dwOldLength] == '_')
                            {
                                bReplace = false;
                            }
                        }

                        if (bReplace == true)
                        {
                            for (unsigned long v = 0; v < dwNewLength; ++v,++dwBufferPos)
                            {
                                pNewData[dwBufferPos] = strNew.m_pString[v];
                            }
                            dwCurrentPos += dwOldLength;
                        }
                        else
                        {
                            for (unsigned long it = 0; it < dwOldLength; ++it,++dwCurrentPos,++dwBufferPos)
                            {
                                pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                            }
                        }
                    }
                    for (; dwCurrentPos < dwLength; ++dwCurrentPos,++dwBufferPos)
                    {
                        pNewData[dwBufferPos] = m_pString[dwCurrentPos];
                    }
                    pNewData[dwNewTotalLength] = 0;
                    delete [] m_pString;
                    m_pString = pNewData;
                    m_dwBufferSize = dwAlloc;
                    dwReplacedCount = dwFoundCount;
                }
            }
            else
            {
                for (unsigned long i = 0; i < dwFoundCount; ++i)
                {
                    for (unsigned long x = 0; x < dwNewLength; ++x)
                    {
                        m_pString[arFoundIndices[i] + x] = strNew.m_pString[x];
                    }
                }
                dwReplacedCount = dwFoundCount;
            }
        }
    }
    return dwReplacedCount;
}

bool AnsiString::ReplaceAt(unsigned long dwIndex,unsigned long dwCount,char ch) throw()
{
    bool bRet(false);
    const unsigned long dwLength = GetLength();
    if (dwIndex < dwLength)
    {
        const unsigned long dwRemainingLength = dwLength - dwIndex;
        unsigned long dwOldLength(0),dwNewLength(0);
        if (dwCount == 0 || dwCount > dwRemainingLength)
        {
            dwOldLength = dwRemainingLength;
        }
        else
        {
            dwOldLength = dwCount;
        }
        dwNewLength = ch != 0 ? 1 : 0;
        for (unsigned long i = dwIndex,j = dwIndex + dwOldLength; j < dwLength; ++j,++i)
        {
            m_pString[i + dwNewLength] = m_pString[j];
        }
        if (dwNewLength != 0)
        {
            m_pString[dwIndex] = ch;
        }
        m_pString[dwLength + dwNewLength - dwOldLength] = 0;
        bRet = true;
    }
    return bRet;
}

bool AnsiString::ReplaceAt(unsigned long dwIndex,unsigned long dwCount,const char * pString) throw()
{
    bool bRet(false);
    const unsigned long dwLength = GetLength();
    if (dwIndex < dwLength)
    {
        const unsigned long dwNewLength = pString != NULL ? StringLength(pString) : 0;
        const unsigned long dwRemainingLength = dwLength - dwIndex;
        unsigned long dwOldLength(0);
        if (dwCount == 0 || dwCount > dwRemainingLength)
        {
            dwOldLength = dwRemainingLength;
            const unsigned long dwTotalLength = dwIndex + dwNewLength;
            if (dwTotalLength >= m_dwBufferSize)
            {
                const unsigned long dwAllocate = DetermineBufferSize(dwTotalLength);
                if (SetCapacity(dwAllocate,true) == false)
                {
                    return false;
                }
            }
            for (unsigned long i = 0; i < dwNewLength; ++i)
            {
                m_pString[dwIndex + i] = pString[i];
            }
            m_pString[dwTotalLength] = 0;
            bRet = true;
        }
        else
        {
            dwOldLength = dwCount;
            unsigned long dwAdjustment(0);
            bool bSubtract(false);
            if (dwOldLength == dwNewLength)
            {
                for (unsigned long vx = 0; vx < dwNewLength; ++vx)
                {
                    m_pString[dwIndex + vx] = pString[vx];
                }
                return true;
            }
            else if (dwOldLength < dwNewLength)
            {
                dwAdjustment = dwNewLength - dwOldLength;
            }
            else if (dwOldLength > dwNewLength)
            {
                dwAdjustment = dwOldLength - dwNewLength;
                bSubtract = true;
            }
            unsigned long dwTotalLength(0);
            if (bSubtract == true)
            {
                dwTotalLength = dwLength - dwAdjustment;
            }
            else
            {
                dwTotalLength = dwLength + dwAdjustment;
            }
            
            const unsigned long dwTempAlloc = DetermineBufferSize(dwTotalLength);
            const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
            char * pTempData = new char[dwAlloc];
            if (pTempData != NULL)
            {
                unsigned long dwBufferPos(0),dwCurrentPos(0);
                for (; dwCurrentPos < dwIndex; ++dwCurrentPos,++dwBufferPos)
                {
                    pTempData[dwBufferPos] = m_pString[dwCurrentPos];
                }
                dwCurrentPos += dwOldLength;
                for (unsigned long it = 0; it < dwNewLength; ++it,++dwBufferPos)
                {
                    pTempData[dwBufferPos] = pString[it];
                }
                for (; dwCurrentPos < dwLength; ++dwCurrentPos,++dwBufferPos)
                {
                    pTempData[dwBufferPos] = m_pString[dwCurrentPos];
                }
                pTempData[dwTotalLength] = 0;
                delete [] m_pString;
                m_pString = pTempData;
                m_dwBufferSize = dwAlloc;
                bRet = true;
            }
        }
    }
    return bRet;
}

bool AnsiString::ReplaceAt(unsigned long dwIndex,unsigned long dwCount,const AnsiString & str) throw()
{
    bool bRet(false);
    const unsigned long dwLength = GetLength();
    if (dwIndex < dwLength)
    {
        const unsigned long dwNewLength = str.GetLength();
        const unsigned long dwRemainingLength = dwLength - dwIndex;
        unsigned long dwOldLength(0);
        if (dwCount == 0 || dwCount > dwRemainingLength)
        {
            dwOldLength = dwRemainingLength;
            const unsigned long dwTotalLength = dwIndex + dwNewLength;
            if (dwTotalLength >= m_dwBufferSize)
            {
                const unsigned long dwAllocate = DetermineBufferSize(dwTotalLength);
                if (SetCapacity(dwAllocate,true) == false)
                {
                    return false;
                }
            }
            for (unsigned long i = 0; i < dwNewLength; ++i)
            {
                m_pString[dwIndex + i] = str.m_pString[i];
            }
            m_pString[dwTotalLength] = 0;
            bRet = true;
        }
        else
        {
            dwOldLength = dwCount;
            unsigned long dwAdjustment(0);
            bool bSubtract(false);
            if (dwOldLength == dwNewLength)
            {
                for (unsigned long vx = 0; vx < dwNewLength; ++vx)
                {
                    m_pString[dwIndex + vx] = str.m_pString[vx];
                }
                return true;
            }
            else if (dwOldLength < dwNewLength)
            {
                dwAdjustment = dwNewLength - dwOldLength;
            }
            else if (dwOldLength > dwNewLength)
            {
                dwAdjustment = dwOldLength - dwNewLength;
                bSubtract = true;
            }
            unsigned long dwTotalLength(0);
            if (bSubtract == true)
            {
                dwTotalLength = dwLength - dwAdjustment;
            }
            else
            {
                dwTotalLength = dwLength + dwAdjustment;
            }
            
            const unsigned long dwTempAlloc = DetermineBufferSize(dwTotalLength);
            const unsigned long dwAlloc = dwTempAlloc > m_dwBufferSize ? dwTempAlloc : m_dwBufferSize;
            char * pTempData = new char[dwAlloc];
            if (pTempData != NULL)
            {
                unsigned long dwBufferPos(0),dwCurrentPos(0);
                for (; dwCurrentPos < dwIndex; ++dwCurrentPos,++dwBufferPos)
                {
                    pTempData[dwBufferPos] = m_pString[dwCurrentPos];
                }
                dwCurrentPos += dwOldLength;
                for (unsigned long it = 0; it < dwNewLength; ++it,++dwBufferPos)
                {
                    pTempData[dwBufferPos] = str.m_pString[it];
                }
                for (; dwCurrentPos < dwLength; ++dwCurrentPos,++dwBufferPos)
                {
                    pTempData[dwBufferPos] = m_pString[dwCurrentPos];
                }
                pTempData[dwTotalLength] = 0;
                delete [] m_pString;
                m_pString = pTempData;
                m_dwBufferSize = dwAlloc;
                bRet = true;
            }
        }
    }
    return bRet;
}

unsigned long AnsiString::Remove(char chRemove) throw()
{
    return Replace(chRemove,0);
}

unsigned long AnsiString::Remove(const char * pstrRemove) throw()
{
    return Replace(pstrRemove,NULL);
}

unsigned long AnsiString::Remove(const AnsiString & strRemove) throw()
{
    return Replace(strRemove,AnsiString());
}

unsigned long AnsiString::Remove(const char * pstrRemove,bool bWholeWordOnly) throw()
{
    return Replace(pstrRemove,NULL,bWholeWordOnly);
}

unsigned long AnsiString::Remove(const AnsiString & strRemove,bool bWholeWordOnly) throw()
{
    return Replace(strRemove,AnsiString(),bWholeWordOnly);
}

bool AnsiString::RemoveAt(unsigned long dwIndex,unsigned long dwCount) throw()
{
    bool bRet(false);
    const unsigned long dwLength = GetLength();
    if (dwIndex < dwLength)
    {
        const unsigned long dwRemainingLength = dwLength - dwIndex;
        if (dwCount == 0 || dwCount > dwRemainingLength)
        {
            m_pString[dwIndex] = 0;
            bRet = true;
        }
        else
        {
            for (unsigned long i = dwIndex,j = dwIndex + dwCount; j < dwLength; ++j,++i)
            {
                m_pString[i] = m_pString[j];
            }
            m_pString[dwLength - dwCount] = 0;
            bRet = true;
        }
    }
    return bRet;
}

void AnsiString::MakeUppercase() throw()
{
    const char chDif = 'A' - 'a';
    const unsigned long dwLength = GetLength();
    for (unsigned long i = 0; i < dwLength; ++i)
    {
        if (m_pString[i] >= 'a' && m_pString[i] <= 'z')
        {
            m_pString[i] += chDif;
        }
    }
}

void AnsiString::MakeLowercase() throw()
{
    const char chDif = 'A' - 'a';
    const unsigned long dwLength = GetLength();
    for (unsigned long i = 0; i < dwLength; ++i)
    {
        if (m_pString[i] >= 'A' && m_pString[i] <= 'Z')
        {
            m_pString[i] -= chDif;
        }
    }
}

void AnsiString::SwitchCase() throw()
{
    const char chDif = 'A' - 'a';
    const unsigned long dwLength = GetLength();
    for (unsigned long i = 0; i < dwLength; ++i)
    {
        if (m_pString[i] >= 'A' && m_pString[i] <= 'Z')
        {
            m_pString[i] -= chDif;
        }
        else if (m_pString[i] >= 'a' && m_pString[i] <= 'z')
        {
            m_pString[i] += chDif;
        }
    }
}

void AnsiString::Reverse() throw()
{
    const unsigned long dwLength = GetLength();
    if (dwLength != 0)
    {
        char chTemp(0);
        for (unsigned long x = 0,y = dwLength - 1; x < y; ++x,--y)
        {
            chTemp = m_pString[x];
            m_pString[x] = m_pString[y];
            m_pString[y] = chTemp;
        }
    }
}

bool AnsiString::SubString(unsigned long dwStart,unsigned long dwCount,AnsiString & strBuffer) const throw()
{
    bool bRet(false);
    const unsigned long dwLength = GetLength();
    if (dwStart < dwLength)
    {
        const unsigned long dwRemainingLength = dwLength - dwStart;
        unsigned long dwCopyLength(0);
        if (dwCount == 0 || dwCount > dwRemainingLength)
        {
            dwCopyLength = dwRemainingLength;
        }
        else
        {
            dwCopyLength = dwCount;
        }
        const unsigned long dwAllocate = DetermineBufferSize(dwCopyLength);
        if (strBuffer.SetCapacity(dwAllocate,false) == true)
        {
            StringCopy(strBuffer.m_pString,&m_pString[dwStart],dwCopyLength);
            strBuffer.m_pString[dwCopyLength] = 0;
            bRet = true;
        }
    }
    return bRet;
}

bool AnsiString::Extract(unsigned long dwStart,unsigned long dwCount,AnsiString & strBuffer) throw()
{
    bool bRet(false);
    const unsigned long dwLength = GetLength();
    if (dwStart < dwLength)
    {
        const unsigned long dwRemainingLength = dwLength - dwStart;
        unsigned long dwCopyLength(0);
        if (dwCount == 0 || dwCount > dwRemainingLength)
        {
            dwCopyLength = dwRemainingLength;
        }
        else
        {
            dwCopyLength = dwCount;
        }
        const unsigned long dwAllocate = DetermineBufferSize(dwCopyLength);
        if (strBuffer.SetCapacity(dwAllocate,false) == true)
        {
            StringCopy(strBuffer.m_pString,&m_pString[dwStart],dwCopyLength);
            strBuffer.m_pString[dwCopyLength] = 0;
            const unsigned long dwCheckIndex = dwStart + dwCopyLength;
            if (dwCheckIndex == dwLength)
            {
                m_pString[0] = 0;
            }
            else
            {
                for (unsigned long i = dwCheckIndex,j = 0; i < dwLength; ++i,++j)
                {
                    m_pString[j] = m_pString[i];
                }
                m_pString[dwLength - dwCopyLength] = 0;
            }
            bRet = true;
        }
    }
    return bRet;
}

bool AnsiString::SpanIncluding(const char * pstrCharSet,AnsiString & strBuffer) const throw()
{
    const AnsiString strCharSet(pstrCharSet);
    const unsigned long dwCharSetLength = strCharSet.GetLength();
    const unsigned long dwLength = GetLength();
    if (dwCharSetLength != 0 && dwLength != 0)
    {
        unsigned long dwCount(0),dwFoundIndex(0);
        for (; dwCount < dwLength; ++dwCount)
        {
            if (strCharSet.Find(m_pString[dwCount],dwFoundIndex) == false)
            {
                break;
            }
        }
        const unsigned long dwBufferSize = DetermineBufferSize(dwCount);
        if (strBuffer.SetCapacity(dwBufferSize,false) == true)
        {
            StringCopy(strBuffer.m_pString,m_pString,dwCount);
            strBuffer.m_pString[dwCount] = 0;
            return true;
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

bool AnsiString::SpanIncluding(const AnsiString & strCharSet,AnsiString & strBuffer) const throw()
{
    const unsigned long dwCharSetLength = strCharSet.GetLength();
    const unsigned long dwLength = GetLength();
    if (dwCharSetLength != 0 && dwLength != 0)
    {
        unsigned long dwCount(0),dwFoundIndex(0);
        for (; dwCount < dwLength; ++dwCount)
        {
            if (strCharSet.Find(m_pString[dwCount],dwFoundIndex) == false)
            {
                break;
            }
        }
        const unsigned long dwAllocate = DetermineBufferSize(dwCount);
        if (strBuffer.SetCapacity(dwAllocate,false) == true)
        {
            StringCopy(strBuffer.m_pString,m_pString,dwCount);
            strBuffer.m_pString[dwCount] = 0;
            return true;
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

bool AnsiString::SpanIncluding(unsigned long dwStart,const char * pstrCharSet,AnsiString & strBuffer) const throw()
{
    const unsigned long dwLength = GetLength();
    if (dwStart < dwLength)
    {
        const AnsiString strCharSet(pstrCharSet);
        const unsigned long dwCharSetLen = strCharSet.GetLength();
        if (dwCharSetLen != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (unsigned long x = dwStart; x < dwLength; ++x)
            {
                if (strCharSet.Find(&m_pString[x],dwFoundIndex) == true)
                {
                    ++dwCount;
                }
                else
                {
                    break;
                }
            }
            const unsigned long dwAllocate = DetermineBufferSize(dwCount);
            if (strBuffer.SetCapacity(dwAllocate,false) == true)
            {
                StringCopy(strBuffer.m_pString,&m_pString[dwStart],dwCount);
                strBuffer.m_pString[dwCount] = 0;
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

bool AnsiString::SpanIncluding(unsigned long dwStart,const AnsiString & strCharSet,AnsiString & strBuffer) const throw()
{
    const unsigned long dwLength = GetLength();
    if (dwStart < dwLength)
    {
        const unsigned long dwCharSetLen = strCharSet.GetLength();
        if (dwCharSetLen != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (unsigned long x = dwStart; x < dwLength; ++x)
            {
                if (strCharSet.Find(&m_pString[x],dwFoundIndex) == true)
                {
                    ++dwCount;
                }
                else
                {
                    break;
                }
            }
            const unsigned long dwAllocate = DetermineBufferSize(dwCount);
            if (strBuffer.SetCapacity(dwAllocate,false) == true)
            {
                StringCopy(strBuffer.m_pString,&m_pString[dwStart],dwCount);
                strBuffer.m_pString[dwCount] = 0;
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

bool AnsiString::SpanExcluding(const char * pstrCharSet,AnsiString & strBuffer) const throw()
{
    const AnsiString strCharSet(pstrCharSet);
    const unsigned long dwCharSetLength = strCharSet.GetLength();
    const unsigned long dwLength = GetLength();
    if (dwCharSetLength != 0 && dwLength != 0)
    {
        unsigned long dwCount(0),dwFoundIndex(0);
        for (; dwCount < dwLength; ++dwCount)
        {
            if (strCharSet.Find(m_pString[dwCount],dwFoundIndex) == true)
            {
                break;
            }
        }
        const unsigned long dwBufferSize = DetermineBufferSize(dwCount);
        if (strBuffer.SetCapacity(dwBufferSize,false) == true)
        {
            StringCopy(strBuffer.m_pString,m_pString,dwCount);
            strBuffer.m_pString[dwCount] = 0;
            return true;
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

bool AnsiString::SpanExcluding(const AnsiString & strCharSet,AnsiString & strBuffer) const throw()
{
    const unsigned long dwCharSetLength = strCharSet.GetLength();
    const unsigned long dwLength = GetLength();
    if (dwCharSetLength != 0 && dwLength != 0)
    {
        unsigned long dwCount(0),dwFoundIndex(0);
        for (; dwCount < dwLength; ++dwCount)
        {
            if (strCharSet.Find(m_pString[dwCount],dwFoundIndex) == true)
            {
                break;
            }
        }
        const unsigned long dwBufferSize = DetermineBufferSize(dwCount);
        if (strBuffer.SetCapacity(dwBufferSize,false) == true)
        {
            StringCopy(strBuffer.m_pString,m_pString,dwCount);
            strBuffer.m_pString[dwCount] = 0;
            return true;
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

bool AnsiString::SpanExcluding(unsigned long dwStart,const char * pstrCharSet,AnsiString & strBuffer) const throw()
{
    const unsigned long dwLength = GetLength();
    if (dwStart < dwLength)
    {
        const AnsiString strCharSet(pstrCharSet);
        const unsigned long dwCharSetLen = strCharSet.GetLength();
        if (dwCharSetLen != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (unsigned long x = dwStart; x < dwLength; ++x)
            {
                if (strCharSet.Find(&m_pString[x],dwFoundIndex) == false)
                {
                    ++dwCount;
                }
                else
                {
                    break;
                }
            }
            const unsigned long dwAllocate = DetermineBufferSize(dwCount);
            if (strBuffer.SetCapacity(dwAllocate,false) == true)
            {
                StringCopy(strBuffer.m_pString,&m_pString[dwStart],dwCount);
                strBuffer.m_pString[dwCount] = 0;
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

bool AnsiString::SpanExcluding(unsigned long dwStart,const AnsiString & strCharSet,AnsiString & strBuffer) const throw()
{
    const unsigned long dwLength = GetLength();
    if (dwStart < dwLength)
    {
        const unsigned long dwCharSetLen = strCharSet.GetLength();
        if (dwCharSetLen != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (unsigned long x = dwStart; x < dwLength; ++x)
            {
                if (strCharSet.Find(&m_pString[x],dwFoundIndex) == false)
                {
                    ++dwCount;
                }
                else
                {
                    break;
                }
            }
            const unsigned long dwAllocate = DetermineBufferSize(dwCount);
            if (strBuffer.SetCapacity(dwAllocate,false) == true)
            {
                StringCopy(strBuffer.m_pString,&m_pString[dwStart],dwCount);
                strBuffer.m_pString[dwCount] = 0;
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

void AnsiString::Scramble() throw()
{
    const unsigned long dwLength = GetLength();
    if (dwLength != 0)
    {
        const char chBits[8] = { 64,16,2,-128,1,4,32,8 };
        char chBit(0),chTemp(0);
        for (unsigned long x = 0; x < dwLength; ++x)
        {
            chBit = 1;
            chTemp = m_pString[x];
            m_pString[x] = 0;
            for (unsigned long i = 0; i < 8; ++i)
            {
                if (chTemp & chBit)
                {
                    m_pString[x] |= chBits[i];
                }
                chBit <<= 1;
            }
        }
    }
}

void AnsiString::Descramble() throw()
{
    const unsigned long dwLength = GetLength();
    if (dwLength != 0)
    {
        const char chBits[8] = { 16,4,32,-128,2,64,1,8 };
        char chBit(0),chTemp(0);
        for (unsigned long x = 0; x < dwLength; ++x)
        {
            chBit = 1;
            chTemp = m_pString[x];
            m_pString[x] = 0;
            for (unsigned long i = 0; i < 8; ++i)
            {
                if (chTemp & chBit)
                {
                    m_pString[x] |= chBits[i];
                }
                chBit <<= 1;
            }
        }
    }
}

void AnsiString::TrimLeft(char chTrim) throw()
{
    if (chTrim != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength != 0)
        {
            unsigned long dwCount(0);
            for (; dwCount < dwLength; ++dwCount)
            {
                if (m_pString[dwCount] != chTrim)
                {
                    break;
                }
            }
            if (dwCount != 0)
            {
                if (dwCount == dwLength)
                {
                    m_pString[0] = 0;
                }
                else
                {
                    for (unsigned long i = dwCount,j = 0; i < dwLength; ++i,++j)
                    {
                        m_pString[j] = m_pString[i];
                    }
                    m_pString[dwLength - dwCount] = 0;
                }
            }
        }
    }
}

void AnsiString::TrimLeft(const char * pstrCharSet) throw()
{
    const AnsiString strCharSet(pstrCharSet);
    const unsigned long dwCharSetLen = strCharSet.GetLength();
    if (dwCharSetLen != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (; dwCount < dwLength; ++dwCount)
            {
                if (strCharSet.Find(m_pString[dwCount],dwFoundIndex) == false)
                {
                    break;
                }
            }
            if (dwCount != 0)
            {
                if (dwCount == dwLength)
                {
                    m_pString[0] = 0;
                }
                else
                {
                    for (unsigned long i = dwCount,j = 0; i < dwLength; ++i,++j)
                    {
                        m_pString[j] = m_pString[i];
                    }
                    m_pString[dwLength - dwCount] = 0;
                }
            }
        }
    }
}

void AnsiString::TrimLeft(const AnsiString & strCharSet) throw()
{
    const unsigned long dwCharSetLen = strCharSet.GetLength();
    if (dwCharSetLen != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (; dwCount < dwLength; ++dwCount)
            {
                if (strCharSet.Find(m_pString[dwCount],dwFoundIndex) == false)
                {
                    break;
                }
            }
            if (dwCount != 0)
            {
                if (dwCount == dwLength)
                {
                    m_pString[0] = 0;
                }
                else
                {
                    for (unsigned long i = dwCount,j = 0; i < dwLength; ++i,++j)
                    {
                        m_pString[j] = m_pString[i];
                    }
                    m_pString[dwLength - dwCount] = 0;
                }
            }
        }
    }
}

void AnsiString::TrimLeft() throw()
{
    TrimLeft(" \t\r\n");
}

void AnsiString::TrimRight(char chTrim) throw()
{
    if (chTrim != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength != 0)
        {
            unsigned long dwCount(0);
            for (unsigned long v = dwLength; v > 0; --v,++dwCount)
            {
                if (m_pString[v - 1] != chTrim)
                {
                    break;
                }
            }
            if (dwCount != 0)
            {
                m_pString[dwLength - dwCount] = 0;
            }
        }
    }
}

void AnsiString::TrimRight(const char * pstrCharSet) throw()
{
    const AnsiString strCharSet(pstrCharSet);
    const unsigned long dwCharSetLen = strCharSet.GetLength();
    if (dwCharSetLen != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (unsigned long v = dwLength; v > 0; --v,++dwCount)
            {
                if (strCharSet.Find(m_pString[v - 1],dwFoundIndex) == false)
                {
                    break;
                }
            }
            if (dwCount != 0)
            {
                m_pString[dwLength - dwCount] = 0;
            }
        }
    }
}

void AnsiString::TrimRight(const AnsiString & strCharSet) throw()
{
    const unsigned long dwCharSetLen = strCharSet.GetLength();
    if (dwCharSetLen != 0)
    {
        const unsigned long dwLength = GetLength();
        if (dwLength != 0)
        {
            unsigned long dwCount(0),dwFoundIndex(0);
            for (unsigned long v = dwLength; v > 0; --v,++dwCount)
            {
                if (strCharSet.Find(m_pString[v - 1],dwFoundIndex) == false)
                {
                    break;
                }
            }
            if (dwCount != 0)
            {
                m_pString[dwLength - dwCount] = 0;
            }
        }
    }
}

void AnsiString::TrimRight() throw()
{
    TrimRight(" \t\r\n");
}

UnicodeString AnsiString::ToUnicode() const throw()
{
    const unsigned long dwLength = GetLength();
    const unsigned long dwAllocate = DetermineBufferSize(dwLength);
    UnicodeString strRetValue(static_cast<unsigned long>(dwAllocate));
    if (strRetValue.m_dwBufferSize == dwAllocate)
    {
        const long nWritten = ::MultiByteToWideChar(CP_ACP,MB_PRECOMPOSED,m_pString,dwLength,strRetValue.m_pString,dwAllocate);
        strRetValue.m_pString[nWritten] = 0;
    }
    return strRetValue;
}

const AnsiString & AnsiString::operator = (char ch) throw()
{
    if (m_pString != NULL)
    {
        m_pString[0] = ch;
        m_pString[1] = 0;
    }
    else
    {
        if (SetCapacity(16,false) == true)
        {
            m_pString[0] = ch;
            m_pString[1] = 0;
        }
    }
    return *this;
}

const AnsiString & AnsiString::operator = (const char * pString) throw()
{
    const unsigned long dwLength = pString != NULL ? StringLength(pString) : 0;
    if (dwLength >= m_dwBufferSize)
    {
        const unsigned long dwAllocate = DetermineBufferSize(dwLength);
        if (SetCapacity(dwAllocate,false) == false)
        {
            return *this;
        }
    }
    StringCopy(m_pString,pString,dwLength);
    m_pString[dwLength] = 0;
    return *this;
}

const AnsiString & AnsiString::operator = (const AnsiString & str) throw()
{
    if (this != &str)
    {
        const unsigned long dwNewLength = str.GetLength();
        const unsigned long dwBufferSize = DetermineBufferSize(dwNewLength);
        if (SetCapacity(dwBufferSize,false) == true)
        {
            StringCopy(m_pString,str.m_pString,dwNewLength);
            m_pString[dwNewLength] = 0;
        }
    }
    return *this;
}

AnsiString & AnsiString::operator += (char ch) throw()
{
    if (ch != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + 1;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwBufferSize = DetermineBufferSize(dwNewLength);
            if (SetCapacity(dwBufferSize,true) == false)
            {
                return *this;
            }
        }
        m_pString[dwLength] = ch;
        m_pString[dwNewLength] = 0;
    }
    return *this;
}

AnsiString & AnsiString::operator += (const char * pString) throw()
{
    const unsigned long dwAddLength = pString != NULL ? StringLength(pString) : 0;
    if (dwAddLength != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwAddLength;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAlloc = DetermineBufferSize(dwNewLength);
            if (SetCapacity(dwAlloc,true) == false)
            {
                return *this;
            }
        }
        StringCopy(&m_pString[dwLength],pString,dwAddLength);
        m_pString[dwNewLength] = 0;
    }
    return *this;
}

AnsiString & AnsiString::operator += (const AnsiString & str) throw()
{
    const unsigned long dwAddLength = str.GetLength();
    if (dwAddLength != 0)
    {
        const unsigned long dwLength = GetLength();
        const unsigned long dwNewLength = dwLength + dwAddLength;
        if (dwNewLength >= m_dwBufferSize)
        {
            const unsigned long dwAlloc = DetermineBufferSize(dwNewLength);
            if (SetCapacity(dwAlloc,true) == false)
            {
                return *this;
            }
        }
        StringCopy(&m_pString[dwLength],str.m_pString,dwAddLength);
        m_pString[dwNewLength] = 0;
    }
    return *this;
}

AnsiString AnsiString::operator + (char ch) const throw()
{
    const unsigned long dwLength = GetLength();
    const unsigned long dwNewLength = dwLength + 1;
    const unsigned long dwBufferSize = DetermineBufferSize(dwNewLength);
    AnsiString strRetValue(static_cast<unsigned long>(dwBufferSize));
    if (strRetValue.m_dwBufferSize == dwBufferSize)
    {
        StringCopy(strRetValue.m_pString,m_pString,dwLength);
        strRetValue.m_pString[dwLength] = ch;
        strRetValue.m_pString[dwNewLength] = 0;
    }
    return strRetValue;
}

AnsiString AnsiString::operator + (const char * pString) const throw()
{
    const unsigned long dwLength = GetLength();
    const unsigned long dwAddLength = pString != NULL ? StringLength(pString) : 0;
    const unsigned long dwNewLength = dwLength + dwAddLength;
    const unsigned long dwBufferSize = DetermineBufferSize(dwNewLength);
    AnsiString strRetValue(static_cast<unsigned long>(dwBufferSize));
    if (strRetValue.m_dwBufferSize == dwBufferSize)
    {
        StringCopy(strRetValue.m_pString,m_pString,dwLength);
        StringCopy(&strRetValue.m_pString[dwLength],pString,dwAddLength);
        strRetValue.m_pString[dwNewLength] = 0;
    }
    return strRetValue;
}

AnsiString AnsiString::operator + (const AnsiString & str) const throw()
{
    const unsigned long dwLength = GetLength();
    const unsigned long dwAddLength = str.GetLength();
    const unsigned long dwNewLength = dwLength + dwAddLength;
    const unsigned long dwBufferSize = DetermineBufferSize(dwNewLength);
    AnsiString strRetValue(static_cast<unsigned long>(dwBufferSize));
    if (strRetValue.m_dwBufferSize == dwBufferSize)
    {
        StringCopy(strRetValue.m_pString,m_pString,dwLength);
        StringCopy(&strRetValue.m_pString[dwLength],str.m_pString,dwAddLength);
        strRetValue.m_pString[dwNewLength] = 0;
    }
    return strRetValue;
}

bool AnsiString::operator == (char ch) const throw()
{
    if (ch == 0)
    {
        return m_pString == NULL || m_pString[0] == 0;
    }
    else
    {
        return m_pString != NULL && m_pString[0] == ch && m_pString[1] == 0;
    }
}

bool AnsiString::operator == (const char * pString) const throw()
{
    const unsigned long dwCompareLength = pString != NULL ? StringLength(pString) : 0;
    const unsigned long dwLength = GetLength();
    if (dwLength == dwCompareLength)
    {
        bool bSame = true;
        for (unsigned long i = 0; i < dwLength; ++i)
        {
            if (m_pString[i] != pString[i])
            {
                bSame = false;
                break;
            }
        }
        return bSame;
    }
    else
    {
        return false;
    }
}

bool AnsiString::operator == (const AnsiString & str) const throw()
{
    if (this != &str)
    {
        const unsigned long dwCompareLength = str.GetLength();
        const unsigned long dwLength = GetLength();
        if (dwLength == dwCompareLength)
        {
            bool bSame = true;
            for (unsigned long i = 0; i < dwLength; ++i)
            {
                if (m_pString[i] != str.m_pString[i])
                {
                    bSame = false;
                    break;
                }
            }
            return bSame;
        }
        else
        {
            return false;
        }
    }
    else
    {
        return true;
    }
}

bool AnsiString::operator != (char ch) const throw()
{
    if (ch == 0)
    {
        return m_pString != NULL && m_pString[0] != 0;
    }
    else
    {
        return m_pString == NULL || m_pString[0] != ch || m_pString[1] != 0;
    }
}

bool AnsiString::operator != (const char * pString) const throw()
{
     const unsigned long dwCompareLength = pString != NULL ? StringLength(pString) : 0;
     const unsigned long dwLength = GetLength();
     if (dwCompareLength == dwLength)
     {
         bool bDifferent = false;
         for (unsigned long i = 0; i < dwLength; ++i)
         {
             if (m_pString[i] != pString[i])
             {
                 bDifferent = true;
                 break;
             }
         }
         return bDifferent;
     }
     else
     {
         return true;
     }
}

bool AnsiString::operator != (const AnsiString & str) const throw()
{
    if (this != &str)
    {
        const unsigned long dwCompareLength = str.GetLength();
        const unsigned long dwLength = GetLength();
        if (dwCompareLength != dwLength)
        {
            return true;
        }
        else
        {
            bool bDifferent = false;
            for (unsigned long x = 0; x < dwLength; ++x)
            {
                if (m_pString[x] != str.m_pString[x])
                {
                    bDifferent = true;
                    break;
                }
            }
            return bDifferent;
        }
    }
    else
    {
        return false;
    }
}

const char & AnsiString::operator [] (unsigned long dwIndex) const throw()
{
    return m_pString[dwIndex];
}

char & AnsiString::operator [] (unsigned long dwIndex) throw()
{
    return m_pString[dwIndex];
}

bool AnsiString::IsCharAlphabetic(char ch) throw()
{
    return (ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z');
}

bool AnsiString::IsCharDigit(char ch) throw()
{
    return ch >= '0' && ch <= '9';
}

bool AnsiString::IsCharAlphaDigit(char ch) throw()
{
    return (ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z') || (ch >= '0' && ch <= '9');
}

bool AnsiString::IsCharWhitespace(char ch,bool bIncludeCrLf) throw()
{
    bool bRet = ch == ' ' || ch == '\t';
    if (bRet == false && bIncludeCrLf == true)
    {
        bRet = ch == '\r' || ch == '\n';
    }
    return bRet;
}

unsigned long AnsiString::DetermineBufferSize(unsigned long dwCharCount) throw()
{
    unsigned long dwTempCount = dwCharCount + 1;
    unsigned long dwBlockCount = dwTempCount / 4096;
    if (dwBlockCount == 0 || (dwTempCount % 4096) != 0)
    {
        ++dwBlockCount;
    }
    const unsigned long dwSizes[] = { 16, 32, 64, 128, 256, 512, 1024, 2048, dwBlockCount * 4096 };
    unsigned long dwIter(0);
    while (dwTempCount > dwSizes[dwIter])
    {
        ++dwIter;
    }
    return dwSizes[dwIter];
}

unsigned long AnsiString::StringLength(const char * pString) throw()
{
    unsigned long dwLength(0);
    while (pString[dwLength] != 0)
    {
         ++dwLength;
    }
    return dwLength;
}

void AnsiString::StringCopy(char * pstrBuffer,const char * pString,unsigned long dwCount) throw()
{
    for (unsigned long i = 0; i < dwCount; ++i)
    {
        pstrBuffer[i] = pString[i];
    }
}

}